package geometry.part6;

import geometry.part2.LineSegment;

import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.List;

public class ConvexPolygon {
    private List<Point2D> vertices; // 頂点
    private List<LineSegment> edges; // 辺

    // verticesには凸多角形の頂点座標が順番に格納してあるものとする
    public ConvexPolygon(List<Point2D> vertices) {
        int size = vertices.size();
        if (size < 3) { // 角数が3未満の場合はエラー
            throw new IllegalArgumentException();
        }
        this.vertices = vertices;
        edges = new ArrayList<LineSegment>();

        // 基準となるCCW値を計算
        double ccw0 = GeomUtils.ccw(vertices.get(0), vertices.get(1), vertices.get(2));
        if (ccw0 == 0) { // ゼロの場合はエラー
            throw new IllegalArgumentException("Polygon is not convex.");
        }
        for (int i = 1; i < size; i++) {
            Point2D v1 = vertices.get(i); // i番目の頂点
            Point2D v2 = vertices.get((i + 1) % size); // v1の次の頂点
            Point2D v3 = vertices.get((i + 2) % size); // v2の次の頂点
            double ccw = GeomUtils.ccw(v1, v2, v3); // CCW値を計算
            if (ccw0 * ccw <= 0) { // 基準値と符号が異なる、またはゼロの場合はエラー
                throw new IllegalArgumentException("Polygon is not convex.");
            }
        }

        for (int i = 0; i < size; i++) {
            Point2D v1 = vertices.get(i); // i番目の頂点
            Point2D v2 = vertices.get((i + 1) % size); // v1の次の頂点
            // 2つの頂点から辺の線分を作成して登録
            edges.add(new LineSegment(
                    v1.getX(), v1.getY(), v2.getX(), v2.getY()));
        }
    }

    // 指定位置の頂点を取得
    public Point2D getVertex(int index) {
        return vertices.get(index);
    }

    // 指定位置の辺を取得
    public LineSegment getEdge(int index) {
        return edges.get(index);
    }

    // 辺の数（＝角数）を取得
    public int getEdgeCount() {
        return edges.size();
    }
}
